@@ -1,5 +1,5 @@
 import asyncio
-import os
+import time
 from enum import Enum
 from typing import TYPE_CHECKING
 
@@ -11,13 +11,18 @@ import scipy
 import torch
 import wandb
 
-from machine_learning.recommending.models import SLIM, SVDRecommender
-from machine_learning.recommending.movielens.data import MovieLens25m
+from machine_learning.recommending.utils import wandb_timeit
 
 if TYPE_CHECKING:
     from machine_learning.recommending.interface import RecommenderModuleBase
 
 
+class Session:
+    def __init__(self):
+        self.id = time.time()
+        wandb.log(dict(session_id=self.id))
+
+
 class Feedback(Enum):
     NONE = "Haven't seen it"
     ONE, TWO, THREE, FOUR, FIVE = map(str, range(1, 6))
@@ -35,45 +40,32 @@ def load_model(torch_nn_module, state_dict_path):
 
 
 class MovieMarkdownGenerator:
-    def __init__(
-        self,
-        links_dataframe,
-        movies_dataframe,
-        tmdb_api_token,
-    ):
-        self.links = links_dataframe
-        self.movies_dataframe = movies_dataframe
-        for dataframe in [self.links, self.movies_dataframe]:
-            dataframe.set_index("movieId", inplace=True)
-            min_index = dataframe.index.min()
-            if min_index == 0:
-                pass
-            elif min_index == 1:
-                dataframe.index -= 1
-            else:
-                raise ValueError("Index of dataframe should start with 0 or 1.")
+    def __init__(self, movielens, tmdb_api_token):
+        self.movielens = movielens
         self.tmdb_api_token = tmdb_api_token
+        self.movies_dataframe = movielens["movies"].set_index("movieId")
 
-    def imdb_id(self, movie_id) -> str:
-        imdb_id = self.links["imdbId"].loc[movie_id]
-        return f"{imdb_id:07d}"
+    @staticmethod
+    def imdb_url(imdb_id):
+        return f"https://www.imdb.com/title/tt{imdb_id:07d}"
 
-    def imdb_url(self, movie_id):
-        return f"https://www.imdb.com/title/tt{self.imdb_id(movie_id)}"
-
-    def tmdb_request_url(self, imdb_id):
+    @staticmethod
+    def tmdb_request_url(imdb_id, tmdb_api_token):
         return (
-            f"https://api.themoviedb.org/3/find/tt{imdb_id}"
-            f"?api_key={self.tmdb_api_token}&external_source=imdb_id"
+            f"https://api.themoviedb.org/3/find/tt{imdb_id:07d}"
+            f"?api_key={tmdb_api_token}&external_source=imdb_id"
         )
 
     @staticmethod
     def poster_url_from_tmdb_poster_path(poster_path, size="w200"):
         return f"https://image.tmdb.org/t/p/{size}{poster_path}"
 
-    def __call__(self, movie_id):
-        imdb_id = self.imdb_id(movie_id)
-        tmdb_request_url = self.tmdb_request_url(imdb_id)
+    def __call__(self, item_id):
+        tmdb_id = self.movielens.model_item_ids_to_tmdb_movie_ids(np.array([item_id]))
+        imdb_id = self.movielens.tmdb_movie_ids_to_imdb_movie_ids(tmdb_id)[0]
+        tmdb_request_url = self.tmdb_request_url(
+            imdb_id=imdb_id, tmdb_api_token=self.tmdb_api_token
+        )
         request = requests.get(tmdb_request_url)
         if not request.ok:
             wandb.log(
@@ -85,7 +77,7 @@ class MovieMarkdownGenerator:
                     )
                 )
             )
-            markdown = self.movies_dataframe.loc[movie_id]["movie title"]
+            markdown = self.movies_dataframe.loc[tmdb_id]["movie title"]
             return markdown, ""
 
         movie_results = request.json()["movie_results"][0]
@@ -93,7 +85,7 @@ class MovieMarkdownGenerator:
             poster_path=movie_results["poster_path"]
         )
         title = movie_results["title"]
-        imdb_url = self.imdb_url(movie_id)
+        imdb_url = self.imdb_url(imdb_id=imdb_id)
         markdown = f"""[{title}]({imdb_url})"""
         return markdown, poster_url
 
@@ -111,9 +103,10 @@ class AheadRecommender:
         user_explicit = scipy.sparse.coo_matrix(
             self.user_explicit.fillna(0).values.reshape(1, -1)
         )
-        user_recommendations = self.recommender.online_recommend(
-            user_explicit, n_recommendations=self.recommender.n_items
-        )[0].numpy()
+        with wandb_timeit("online_recommend"):
+            user_recommendations = self.recommender.online_recommend(
+                user_explicit, n_recommendations=self.recommender.n_items
+            )[0].numpy()
         new_items_mask = self.user_explicit.isna().values
         new_items_positions = new_items_mask[user_recommendations].nonzero()[0]
         movie_id = user_recommendations[new_items_positions[0]]
@@ -135,34 +128,41 @@ class AheadRecommender:
                 f"Provided movie id {movie_id} doesn't "
                 f"match stored movie id {self.movie_id}"
             )
-        wandb.log(dict(movie_id=self.movie_id, rating=rating))
         self.user_explicit.loc[self.movie_id] = rating
         self.movie_id, self.next_movie_id = self.next_movie_id, None
 
-    def __del__(self):
-        wandb.log(
-            dict(feedback=wandb.Table(dataframe=self.user_explicit.dropna().to_frame()))
-        )
-
 
 class AsyncRecommendingAppContentDispenser:
-    def __init__(self, recommender: "RecommenderModuleBase", movie_markdown_generator):
+    def __init__(
+        self,
+        recommender: "RecommenderModuleBase",
+        movie_markdown_generator: MovieMarkdownGenerator,
+    ):
         self.ahead_recommender = AheadRecommender(recommender=recommender)
         self.movie_markdown_generator = movie_markdown_generator
         self.movie_id = self.ahead_recommender.movie_id
-        self.initial_content = self.movie_markdown_generator(self.movie_id)
+        self._initial_content = None
+        self.next_content = None
+
+    def initial_content(self):
+        if self._initial_content is not None:
+            return self._initial_content
+        self._initial_content = self.movie_markdown_generator(item_id=self.movie_id)
         self.next_content = asyncio.create_task(self.content_ahead())
 
     async def content_ahead(self):
-        movie_id = self.ahead_recommender.next_recommendation()
-        content = self.movie_markdown_generator(movie_id)
-        return movie_id, content
+        with wandb_timeit("recommending"):
+            movie_id = self.ahead_recommender.next_recommendation()
+        with wandb_timeit("tmdb"):
+            markdown, poster_url = self.movie_markdown_generator(item_id=movie_id)
+        return movie_id, markdown, poster_url
 
-    async def content(self, feedback: Feedback):
+    async def content(self, feedback: Feedback, session: Session):
+        wandb.log(dict(movie_id=self.movie_id, rating=feedback.int, session=session.id))
         self.ahead_recommender.save_rating(movie_id=self.movie_id, rating=feedback.int)
-        self.movie_id, content = await self.next_content
+        self.movie_id, markdown, poster_url = await self.next_content
         self.next_content = asyncio.create_task(self.content_ahead())
-        return content
+        return markdown, poster_url
 
 
 def build_app_blocks(
@@ -173,53 +173,22 @@ def build_app_blocks(
         movie_markdown_generator=movie_markdown_generator,
     )
     gr.Markdown("This movie recommender adapts to your preferences")
+    session = gr.State(value=Session())
     with gr.Row():
-        first_movie_markdown, first_image_url = dispenser.initial_content
-        image = gr.Image(first_image_url, interactive=False)
+        image = gr.Image(lambda: dispenser.initial_content()[1], interactive=False)
         image.style(width=200)
         with gr.Column(scale=4):
-            markdown = gr.Markdown(first_movie_markdown)
+            markdown = gr.Markdown(lambda: dispenser.initial_content()[0])
 
             def build_button(feedback):
-                async def wrapper():
-                    return await dispenser.content(feedback=feedback)
+                async def wrapper(session):
+                    return await dispenser.content(feedback=feedback, session=session)
 
                 button = gr.Button(feedback.value)
-                button.click(fn=wrapper, inputs=[], outputs=[markdown, image])
+                button.click(fn=wrapper, inputs=[session], outputs=[markdown, image])
                 return button
 
             build_button(Feedback.NONE)
             with gr.Row():
                 for rated_feedback in list(Feedback)[1:]:
                     build_button(rated_feedback)
-
-
-def main():
-    project = "Recommending"
-    entity = "dkoshman"
-    path_to_movielens_folder = "local/ml-25m"
-    state_dict_path = "svd.pt"
-    tmdb_api_token = os.environ["TMDB_API_TOKEN"]
-
-    movielens = MovieLens25m(path_to_movielens_folder=path_to_movielens_folder)
-    movie_markdown_generator = MovieMarkdownGenerator(
-        links_dataframe=movielens["links"],
-        movies_dataframe=movielens["movies"],
-        tmdb_api_token=tmdb_api_token,
-    )
-    model = load_model(torch_nn_module=SVDRecommender, state_dict_path=state_dict_path)
-    model.eval()
-
-    with gr.Blocks() as app:
-        build_app_blocks(
-            recommender=model,
-            movie_markdown_generator=movie_markdown_generator,
-        )
-
-    wandb.finish()
-    wandb.init(job_type="app", project=project, entity=entity)
-    app.launch()
-
-
-if __name__ == "__main__":
-    main()
